/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.plugins.index.property;

import java.util.List;

import org.apache.jackrabbit.oak.spi.query.Filter;
import org.apache.jackrabbit.oak.spi.query.QueryIndex;
import org.apache.jackrabbit.oak.spi.state.NodeState;

import com.google.common.collect.ImmutableList;

/**
 * convenience provider for going around the INFINITE cost of the actual index implementation
 */
public class LowCostOrderedPropertyIndexProvider extends OrderedPropertyIndexProvider {
    @Override
    public List<? extends QueryIndex> getQueryIndexes(NodeState nodeState) {
        return ImmutableList.<QueryIndex> of(new LowCostOrderedPropertyIndex(this));
    }

    private static class LowCostOrderedPropertyIndex extends OrderedPropertyIndex {
        public LowCostOrderedPropertyIndex(LowCostOrderedPropertyIndexProvider indexProvider) {
            super(indexProvider);
        }

        @Override
        public double getMinimumCost() {
            return 1e-3;
        }

        @Override
        public double getCost(Filter filter, NodeState root) {
            return getMinimumCost();
        }
    }
}
